"""
Yum plugin for RHN access.

This plugin provides access to Red Hat Network to yum via up2date modules
and XMLRPC calls.
"""

import sys

from yum.plugins import TYPE_CORE
from yum.yumRepo import YumRepository

import yum.Errors

from rhpl.translate import _

# TODO: Get the up2date stuff that we need in a better place,
# so we don't have to do path magic.
sys.path.append("/usr/share/rhn/")

import up2date_client.up2dateAuth as up2dateAuth
from up2date_client import config
from up2date_client import rhnChannel
from up2date_client import rhnPackageInfo
from up2date_client import up2dateErrors

__revision__ = "$Rev: 105058 $"

requires_api_version = '2.5'
plugin_type = TYPE_CORE

rhn_enabled = True

COMMUNICATION_ERROR = _("There was an error communicating with RHN.")

def init_hook(conduit):
    """ 
    Plugin initialization hook. We setup the RHN channels here. 
    
    We get a list of RHN channels from the server, then make a repo object for
    each one. This list of repos is then added to yum's list of repos via the 
    conduit.
    """

    global rhn_enabled
   
    RHN_DISABLED = _("RHN support will be disabled.")

    up2date_cfg = config.initUp2dateConfig()
    try:
        proxy_url = get_proxy_url(up2date_cfg)
        proxy_dict = {'HTTPS' : proxy_url}
    except BadProxyConfig:
        rhn_enabled = False
        PROXY_ERROR =  _("There was an error parsing the RHN proxy settings.") 
        conduit.error(0, PROXY_ERROR + "\n" + RHN_DISABLED)
        return 

    # We might not have an opt parser (ie in pirut)
    opt_parser = conduit.getOptParser()
    if opt_parser:
        (opts, commands) = conduit.getOptParser().parse_args()

        if opts.version or (len(commands) < 1):
            rhn_enabled = False
            conduit.info(10, _("Either --version, or no commands entered") +
                "\n" + RHN_DISABLED)
            return

    try:
        login_info = up2dateAuth.getLoginInfo()
    except up2dateErrors.RhnServerException, e:
        conduit.error(0, COMMUNICATION_ERROR + "\n" + RHN_DISABLED + "\n" +
            str(e))
        rhn_enabled = False
        return

    if not login_info:
        conduit.error(0, _("This system is not registered with RHN.") + "\n" +
            RHN_DISABLED)
        rhn_enabled = False
        return 

    CHANNELS_DISABLED = _("RHN channel support will be disabled.")
    try:
        svrChannels = rhnChannel.getChannelDetails()
    except up2dateErrors.NoChannelsError:
        conduit.error(0, _("This system is not subscribed to any channels.") + 
            "\n" + CHANNELS_DISABLED)
        return
    except up2dateErrors.RhnServerException, e:
        conduit.error(0, COMMUNICATION_ERROR + "\n" + CHANNELS_DISABLED + 
            "\n" + str(e))
        return
   
    repos = conduit.getRepos()
    cachedir = conduit.getConf().cachedir
    default_gpgcheck = conduit.getConf().gpgcheck
    gpgcheck = conduit.confBool('main', 'gpgcheck', default_gpgcheck)
    sslcacert = up2date_cfg['sslCACert']
    for channel in svrChannels:
        if channel['label'] not in rhnChannel.channel_blacklist \
           and channel['version']:
            repo = RhnRepo(channel)
            repo.basecachedir = cachedir
            repo.gpgcheck = gpgcheck
            repo.proxy = proxy_url
            repo.sslcacert = sslcacert
            repos.add(repo)


def posttrans_hook(conduit):
    """ Post rpm transaction hook. We update the RHN profile here. """
    global rhn_enabled
    if rhn_enabled:
        ts_info = conduit.getTsInfo()
        delta = make_package_delta(ts_info)
        try:
            rhnPackageInfo.remoteDeltaPackages(delta)
        except up2dateErrors.RhnServerException, e:
            conduit.error(0, COMMUNICATION_ERROR + "\n" +
                _("Package profile information could not be sent.") + "\n" + 
                str(e))


class RhnRepo(YumRepository):

    """
    Repository object for Red Hat Network.

    This, along with the RhnPackageSack, adapts up2date for use with
    yum.
    """
    rhn_needed_headers = ['X-RHN-Server-Id',
                          'X-RHN-Auth-User-Id',
                          'X-RHN-Auth',
                          'X-RHN-Auth-Server-Time',
                          'X-RHN-Auth-Expire-Offset']
    
    def __init__(self, channel):
        YumRepository.__init__(self, channel['label'])

        # We don't really care what the bit after the channel url is,
        # it's filler

        # TODO: name it something better than this. $RHN is probably legal
        # based upon misa & bretm's reading of the http spec, but urlgrabber 
        # escapes it, and existing server code doesn't automagically unescape 
        # it :/
        url = channel['url'] + '/GET-REQ/'

        self.baseurl = [url + self.id]
        self.urls = self.baseurl
        self.failovermethod = None
        self.keepalive = 0
        self.bandwidth = 0
        self.retries = 1
        self.throttle = 0
        self.timeout = None
        self.metadata_cookie = 'cookie'
        self.metadata_expire = 0

        # No support for package groups
        self.groups_enabled = False

        self.http_caching = True

        self.gpgkey = []
        self.gpgcheck = False
    
        if channel['gpg_key_url'] != '':
            self.gpgkey = [channel['gpg_key_url'],]

        self.enable()

    def setupRhnHttpHeaders(self):
        """ Set up self.http_headers with needed RHN X-RHN-blah headers """
        
        try:
            li = up2dateAuth.getLoginInfo()
        except up2dateErrors.RhnServerException, e:
            raise yum.Errors.RepoError(str(e))

        # TODO:  do evalution on li auth times to see if we need to obtain a
        # new session...

        for header in RhnRepo.rhn_needed_headers:
            if not li.has_key(header):
                error = "Missing required login information for RHN: %s" \
                    % header
                raise yum.Errors.RepoError(error)
            
            self.http_headers[header] = li[header]

    def getPackage(self, package, checkfunc = None, text = None, cache = True):
        """ Download a package from the RHN repository. """
        self.setupRhnHttpHeaders()
        return YumRepository.getPackage(self, package, checkfunc, text, cache)


    def getHeader(self, package, checkfunc = None, reget = 'simple',
                  cache = True):
        """ Download an RPM package header from the RHN repository. """
        self.setupRhnHttpHeaders()
        return YumRepository.getHeader(self, package, checkfunc, reget, cache)
        
    def _loadRepoXML(self, text = None):
        self.setupRhnHttpHeaders()
        YumRepository._loadRepoXML(self, text)

    def retrieveMD(self, mdtype):
        self.setupRhnHttpHeaders()
        return YumRepository.retrieveMD(self, mdtype)

    def getGroupLocation(self):
        """
        Return the location of the package group. We don't support groups so we
        return nothing.
        """
        return None


def make_package_delta(ts_info):
    """ 
    Construct an RHN style package delta from a yum TransactionData object. 
    
    Return a hash containing two keys: added and removed.
    Each key's value is a list of RHN style package tuples.
    """

    delta = {}
    delta["added"] = []
    delta["removed"] = []

    # Make sure the transaction data has the packages in nice lists.
    ts_info.makelists()    

    for ts_member in ts_info.installed:
        package = ts_member.po
        pkgtup = __rhn_pkg_tup_from_po(package)
        delta["added"].append(pkgtup)

    # TODO: are old versions of updated package in the removed list?
    # What happens on the rhn side?
    # updated package might get messed up in the profile
    for ts_member in ts_info.updated:
        package = ts_member.po
        pkgtup = __rhn_pkg_tup_from_po(package)
        delta["added"].append(pkgtup)

    for ts_member in ts_info.removed:
        package = ts_member.po
        pkgtup = __rhn_pkg_tup_from_po(package)
        delta["removed"].append(pkgtup)

    return delta

def __rhn_pkg_tup_from_po(package):
    """ Construct an rhn-style package tuple from a yum package object. """

    name = package.returnSimple('name')
    epoch = package.returnSimple('epoch')
    version = package.returnSimple('version')
    release = package.returnSimple('release')
    arch = package.returnSimple('arch')
    
    return (name, version, release, epoch, arch)


class BadProxyConfig(Exception):
    pass

def get_proxy_url(up2date_cfg):
    if not up2date_cfg['enableProxy']:
        return None

    # Even though we're using HTTP, we have to say we're using
    # HTTPS to trick urllib2.
    proxy_url = 'https://'
    if up2date_cfg['enableProxyAuth']:
        if not up2date_cfg.has_key('proxyUser'):
            raise BadProxyConfig
        if not up2date_cfg.has_key('proxyPassword'):
            raise BadProxyConfig
        proxy_url = proxy_url + up2date_cfg['proxyUser']
        proxy_url = proxy_url + ':'
        proxy_url = proxy_url + up2date_cfg['proxyPassword']
        proxy_url = proxy_url + '@'
   
    netloc = up2date_cfg['httpProxy']
    # Check if a protocol is supplied. We'll ignore it.
    proto_split = netloc.split('://')
    if len(proto_split) > 1:
       netloc = proto_split[1]

    return proxy_url + netloc
