# Copyright (c) 1999-2002 Red Hat, Inc.  Distributed under GPL.
#
# Author: James Bowes <jbowes@redhat.com>
#

import os
import sys

import yum

sys.path.append("/usr/share/yum-cli")

import callback

sys.path.append("/usr/share/rhn/")

from up2date_client import up2dateLog
from up2date_client import up2dateUtils
from up2date_client import config
from up2date_client import rpmUtils
from up2date_client import rhnPackageInfo

log = up2dateLog.initLog()

YUM_PID_FILE = '/var/run/yum.pid'

# mark this module as acceptable
__rhnexport__ = [
    'update',
    'remove',
    'refresh_list',
    'fullUpdate',
    'checkNeedUpdate',
    'runTransaction',
    'verify',
    'verifyAll'
]

class YumAction(yum.YumBase):

    def __init__(self):
        yum.YumBase.__init__(self)

        self.doConfigSetup()

        self.doTsSetup()
        self.doRpmDBSetup()
        self.doRepoSetup()
        self.doSackSetup()

    # Copied from yum/cli.py, more or less
    def doTransaction(self):
        """takes care of package downloading, checking, user confirmation and actually
           RUNNING the transaction"""

        # Check which packages have to be downloaded
        downloadpkgs = []
        for txmbr in self.tsInfo.getMembers():
            if txmbr.ts_state in ['i', 'u']:
                po = txmbr.po
                if po:
                    downloadpkgs.append(po)

        log.log_debug('Downloading Packages:')
        problems = self.downloadPkgs(downloadpkgs) 

        if len(problems.keys()) > 0:
            errstring = ''
            errstring += 'Error Downloading Packages:\n'
            for key in problems.keys():
                errors = yum.misc.unique(problems[key])
                for error in errors:
                    errstring += '  %s: %s\n' % (key, error)
            raise yum.Errors.YumBaseError, errstring

        # Check GPG signatures
        if self.gpgsigcheck(downloadpkgs) != 0:
            return 1
        
        log.log_debug('Running Transaction Test')
        tsConf = {}
        for feature in ['diskspacecheck']: # more to come, I'm sure
            tsConf[feature] = getattr(self.conf, feature)
        
        # clean out the ts b/c we have to give it new paths to the rpms 
        del self.ts
        
        self.initActionTs()
        # save our dsCallback out
        testcb = callback.RPMInstallCallback(output=0)
        testcb.tsInfo = self.tsInfo
        dscb = self.dsCallback
        self.dsCallback = None # dumb, dumb dumb dumb!
        self.populateTs(keepold=0) # sigh
        tserrors = self.ts.test(testcb, conf=tsConf)
        
        log.log_debug('Finished Transaction Test')
        if len(tserrors) > 0:
            errstring = 'Transaction Check Error: '
            for descr in tserrors:
                errstring += '  %s\n' % descr 
            
            raise yum.Errors.YumBaseError, errstring
        log.log_debug('Transaction Test Succeeded')
        del self.ts
        
        self.initActionTs() # make a new, blank ts to populate
        self.populateTs(keepold=0) # populate the ts
        self.ts.check() #required for ordering
        self.ts.order() # order

        # put back our depcheck callback
        self.dsCallback = dscb

        log.log_debug('Running Transaction')
        self.runTransaction(testcb)

        # close things
        return 0

    # Also taken from yum/cli.py
    def gpgsigcheck(self, pkgs):
        '''Perform GPG signature verification on the given packages, installing
        keys if possible

        Returns non-zero if execution should stop (user abort).
        Will raise YumBaseError if there's a problem
        '''
        for po in pkgs:
            result, errmsg = self.sigCheckPkg(po)

            if result == 0:
                # Verified ok, or verify not req'd
                continue            

            elif result == 1:
                raise yum.Errors.YumBaseError, \
                    'Refusing to automatically import keys when running ' \
                    'unattended.\nUse "-y" to override.'

            else:
                # Fatal error
                raise yum.Errors.YumBaseError, errmsg

        return 0


def remove(package_list):
    """We have been told that we should remove packages"""

    if type(package_list) != type([]):
        return (13, "Invalid arguments passed to function", {})

    log.log_debug("Called remove_packages", package_list)

    transaction_data = __make_transaction(package_list, 'e')
    
    return runTransaction(transaction_data)

def update(package_list):        
    """We have been told that we should retrieve/install packages"""
 
    if type(package_list) != type([]):
        return (13, "Invalid arguments passed to function", {})

    log.log_debug("Called update", package_list)
  
    transaction_data = __make_transaction(package_list, 'i')
   
    return runTransaction(transaction_data)

def __make_transaction(package_list, action):
    """
    Build transaction Data like runTransaction would expect.
    This is a list of ((n,v,r,e), m) where m is either e, i, or u
    """

    transaction_data = {}
    transaction_data['packages'] = []

    #We don't care about this stuff.
    transaction_data['flags'] = []
    transaction_data['vsflags'] = []
    transaction_data['probFilterFlags'] = []

    for package in package_list:
        transaction_data['packages'].append((package, action))

    return transaction_data


def runTransaction(transaction_data):
    """ Run a tranaction on a group of packages. """
    
    yum_base = YumAction()
    yum_base.doLock(YUM_PID_FILE)


    for transaction_item in transaction_data['packages']:
        package_tup = yum_package_tup(transaction_item[0])
        action = transaction_item[1]

        package = yum_base.getPackageObject(package_tup) 
       
        if action == 'u':
            yum_base.update(package)
        elif action == 'i':
            yum_base.install(package)
        elif action == 'e':
            yum_base.remove(package)
        else:
            assert False, "Unknown package transaction action."

    yum_base.buildTransaction()
    yum_base.doTransaction()

    yum_base.closeRpmDB()
    yum_base.doUnlock(YUM_PID_FILE)

    return (0, "Transaction Succeeded", {})

def yum_package_tup(package_tup):
    """ Create a yum-style package tuple from an rhn package tuple. """
    return (package_tup[0], '', package_tup[3], package_tup[1], package_tup[2])


def fullUpdate(force=0):
    """ Update all packages on the system. """
    #TODO: force doesn't mean anything for yum.

    yum_base = YumAction()
    yum_base.doLock(YUM_PID_FILE)

    yum_base.update() 
    yum_base.buildTransaction()
    yum_base.doTransaction()

    yum_base.closeRpmDB()
    yum_base.doUnlock(YUM_PID_FILE)


# The following functions are the same as the old up2date ones.

# Check if the locally installed package list changed, if
# needed the list is updated on the server
# In case of error avoid pushing data to stay safe
#
def checkNeedUpdate(rhnsd=None):
    data = {}
    dbpath = "/var/lib/rpm"
    cfg = config.initUp2dateConfig()
    if cfg['dbpath']:
        dbpath = cfg['dbpath']
    RPM_PACKAGE_FILE="%s/Packages" % dbpath

    try:
        dbtime = os.stat(RPM_PACKAGE_FILE)[8] # 8 is st_mtime
    except:
        return (0, "unable to stat the rpm database", data)
    try:
        last = os.stat(up2dateUtils.LAST_UPDATE_FILE)[8]
    except:
        last = 0;

    # Never update the package list more than once every 1/2 hour
    if last >= (dbtime - 10):
        return (0, "rpm database not modified since last update (or package "
            "list recently updated)", data)
    
    if last == 0:
        try:
            file = open(up2dateUtils.LAST_UPDATE_FILE, "w+")
            file.close()
        except:
            return (0, "unable to open the timestamp file", data)

    # call the refresh_list action with a argument so we know it's
    # from rhnsd
    return refresh_list(rhnsd=1)
   
# push again the list of rpm packages to the server
def refresh_list(rhnsd=None):
    log.log_debug("Called refresh_rpmlist")

    ret = None

    try:
        rhnPackageInfo.updatePackageProfile()
    except:
        print "ERROR: refreshing remote package list for System Profile"
        return (20, "Error refreshing package list", {})

    up2dateUtils.touchTimeStamp()
    return (0, "rpmlist refreshed", {})

def verify(packages):
    log.log_debug("Called packages.verify")

    data = {}
    data['name'] = "packages.verify"
    data['version'] = 0
    ret, missing_packages = rpmUtils.verifyPackages(packages)
                                                                                
    data['verify_info'] = ret
    
    if len(missing_packages):
        data['name'] = "packages.verify.missing_packages"
        data['version'] = 0
        data['missing_packages'] = missing_packages
        return(43, "packages requested to be verified are missing", data)

    return (0, "packages verified", data)

def verifyAll():
    log.log_debug("Called packages.verifyAll")

    data = {}
    data['name'] = "packages.verifyAll"
    data['version'] = 0

    ret = rpmUtils.verifyAllPackages()
    data['verify_info'] = ret
    return (0, "packages verified", data)
